#!/usr/bin/perl
use List::Util qw(min);
use CGI;
use POWER::Utils;
use POWER::Tree::AnyValue;
use strict;
use warnings;

my $REQMIN = 49; #  ...

my $div = "="x70;
my $log = `find /var/log/pproxy/ -type f -name '@*' -o -name 'current' |
    sort | xargs sed 's,^[^ ]* ,,'`;
my @req; push @req, [$1, $2, $3]
    while $log =~ /^([^\n]+)\n$div\n((?:GET|POST) .*?)$div\n(.*?)$div\n/msg;

my %Q = CGI::Vars();
my $now = sub {sprintf "%02d/%02d/%04d %02d:%02d:%02d",
    $_[3], $_[4]+1, $_[5]+1900, $_[2], $_[1], $_[0]} -> (localtime);
$Q{From} = $now if $Q{From} and $Q{From} =~ /now/i;
my $from = $Q{From} || $Q{Now} || $now;
@req = grep {$_->[0] ge $from				    } @req;
@req = grep {$_->[1]=~/^(?:GET|POST|Host:).*\Q$Q{InUrl}\E/im } @req if $Q{InUrl};

my $tree = POWER::Tree::AnyValue->root('PProxy log');
my %node;
for (@req[0 .. min($#req, $REQMIN)]) {
    my ($d,$t) = $_->[0] =~ m!^(\d\d/\d\d/\d\d\d\d) (\d\d:\d\d:\d\d)!;
    my ($req) = $_->[1] =~ /^(.*) /;
    $node{$d} ||= $tree->add_value('/', $d);
    my $thisnode = $tree->add_value($node{$d}, "$t $req");
    $tree->add_value($thisnode, [@$_[1,2]]);
#    $node{"$d $h"} ||= $tree->find_node($tree->add_value('/', "$d $h hour"));
#    push @{ $node{"$d $h"} }, [
#	    { ID => ++$tree->{ID}, VALUE => "$t $req" },
#	    [ { ID => ++$tree->{ID}, VALUE => $_ }    ],
#	];
}

print <<EOHEAD;
Content-type: text/html\r
Pragma: no-cache\r
Expires: access\r
\r
<HTML>
<HEAD>
    <TITLE>PProxy LOG</TITLE>
</HEAD>
<BODY BGCOLOR="#eee9e9">
<FORM>
<TABLE>
<TR><TD>From:<TD><INPUT TYPE=TEXT NAME=From VALUE="${\Enc($Q{From}||q{})}" SIZE=20>
    <TD>InUrl:<TD><INPUT TYPE=TEXT NAME=InUrl VALUE="${\Enc($Q{InUrl}||q{})}" SIZE=20>
    <TD>AsIs:<TD><INPUT TYPE=CHECKBOX NAME=AsIs VALUE=1 ${\($Q{AsIs} ? 'CHECKED' : '')}>
    <TD>
    <INPUT TYPE=HIDDEN NAME=Now VALUE="${\Enc($now||q{})}">
    <INPUT TYPE=SUBMIT>
</TABLE>
</FORM>
<H4>PProxy log from: ${\Enc($from||q{})} [${\(1+min($#req, $REQMIN))} requests]</H4>
EOHEAD

my %IMAGES = (
    '  ' => '<IMG BORDER=0 SRC="images/blank.gif" ALT="&nbsp;&nbsp;">'	,
    ' |' => '<IMG BORDER=0 SRC="images/vertline.gif" ALT="&nbsp;|">'	,
    '=o' => '<IMG BORDER=0 SRC="images/snode.gif" ALT="=o">'.
	    '<IMG BORDER=0 SRC="images/doc.gif" ALT="">'		,
    '=+' => '<IMG BORDER=0 SRC="images/spnode.gif" ALT="=+">'.
	    '<IMG BORDER=0 SRC="images/folderclosed.gif" ALT="">'	,
    '=-' => '<IMG BORDER=0 SRC="images/smnode.gif" ALT="=-">'.
	    '<IMG BORDER=0 SRC="images/folderopen.gif" ALT="">'	,
    '~o' => '<IMG BORDER=0 SRC="images/rnode.gif" ALT="~o">'.
	    '<IMG BORDER=0 SRC="images/doc.gif" ALT="">'		,
    '~+' => '<IMG BORDER=0 SRC="images/rpnode.gif" ALT="~+">'.
	    '<IMG BORDER=0 SRC="images/folderclosed.gif" ALT="">'	,
    '~-' => '<IMG BORDER=0 SRC="images/rmnode.gif" ALT="~-">'.
	    '<IMG BORDER=0 SRC="images/folderopen.gif" ALT="">'	,
    ' o' => '<IMG BORDER=0 SRC="images/node.gif" ALT="&nbsp;o">'.
	    '<IMG BORDER=0 SRC="images/doc.gif" ALT="">'		,
    ' +' => '<IMG BORDER=0 SRC="images/pnode.gif" ALT="&nbsp;+">'.
	    '<IMG BORDER=0 SRC="images/folderclosed.gif" ALT="">'	,
    ' -' => '<IMG BORDER=0 SRC="images/mnode.gif" ALT="&nbsp;-">'.
	    '<IMG BORDER=0 SRC="images/folderopen.gif" ALT="">'	,
    'vo' => '<IMG BORDER=0 SRC="images/lastnode.gif" ALT="vo">'.
	    '<IMG BORDER=0 SRC="images/doc.gif" ALT="">'		,
    'v+' => '<IMG BORDER=0 SRC="images/plastnode.gif" ALT="v+">'.
	    '<IMG BORDER=0 SRC="images/folderclosed.gif" ALT="">'	,
    'v-' => '<IMG BORDER=0 SRC="images/mlastnode.gif" ALT="v-">'.
	    '<IMG BORDER=0 SRC="images/folderopen.gif" ALT="">'	,
    );
sub is_open {
    return 1;
    $_[0] =~ m{^/\d*$};
}
for ( @{ $tree->tree2list(\&is_open, 1) } ) {
    my ($path, $data, @prefix) = (pop @$_, pop @$_, @$_);
    my $img = join("", map {$IMAGES{$_}} @prefix);
    print <<EOHTML;
<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0><TR>
EOHTML
    if (not ref($data)) {
	print <<EOHTML;
<TD VALIGN=TOP>$img</TD>
<TD VALIGN=MIDDLE NOWRAP>
<FONT COLOR=RED>${\Enc($data)}</FONT>
EOHTML
    } else {
	my ($request, $reply) = @$data;
	$request = nice($request);
	$reply   = nice($reply);
	my $img2 = join("", map {$IMAGES{$_}} map {m/^ \S/?' |':'  '} @prefix);
	$img2 = join '<BR>', ($img2) x (2 + (() = "$request$reply"=~/<BR>/g));
	$img2 =~ s/<IMG/<IMG WIDTH=16 HEIGHT=18/g;
	print <<EOHTML;
<TR><TD>$img<BR>$img2<TD NOWRAP><FONT COLOR=GREEN STYLE="font-family: monospace; font-size: 14"
    >$request</FONT><BR><BR><FONT COLOR=BLUE STYLE="font-family: monospace; font-size: 14"
    >$reply</FONT>
EOHTML
    }
    print <<EOHTML;
</TABLE>
EOHTML
}



print <<EOTAIL;
</BODY>
</HTML>
EOTAIL

sub nice {
    local $_ = shift;
    s/\r//g;
    s/^(GET .*?\?|POST .*?\?|Cookie:|\n)(.*)/$1._param($2)/gmie;
    s/^(Set-Cookie:\s*)(\S*?=)/$1._param($2)/gmie;
    $_ = Enc($_);
    s{&#1;}{<B><FONT COLOR=MAGENTA>}g; s{&#2;}{</FONT></B>}g;
    s{&#3;}{\n<FONT COLOR=GRAY>[+]</FONT>}g;
    $_.="<FONT COLOR=GRAY>[NO EOL]</FONT>" unless /\n$/;
    s/\n\s*$//;
    s/\n/<BR>/g;
    return $_;
}
sub _param {
    local $_ = shift;
    s{(^|&|; )(.*?)=}{$1\x01$2\x02=}g;
    s{(&|; )(?=.)}{$1\x03}g if length($_) > 100;
    $_ = UnEncUri($_) unless $Q{AsIs};
    return $_;
}

